// fair4ctf.cpp : Defines the entry point for the DLL application.
//

#include "bzfsAPI.h"
#include <algorithm>
#include <cmath>

BZ_GET_PLUGIN_VERSION

class PlayerInfo;

class PlayerInfo {
public:
	int id, grabs;
	double lastGrab;
	PlayerInfo *next;
};

class FCTF : public bz_EventHandler, public bz_CustomSlashCommandHandler {
public:
	FCTF() {
		fairCTFEnabled = true; // turn on/off this plugin
		notifiedCTFOK_RG = false;
		notifiedCTFOK_RB = false;
		notifiedCTFOK_RP = false;
		notifiedCTFOK_GB = false;
		notifiedCTFOK_GP = false;
		notifiedCTFOK_BP = false;
		fairCTF = true;  // all teams are fair = true
		kickEnabled = false; // auto-kick is off by default, uses /kill instead

		RvG = 0;
		RvB = 0;
		RvP = 0;
		GvB = 0;	
		GvP = 0;
		BvP = 0;

		// don't worry about teams with max 0 players
		red = (bz_getTeamPlayerLimit((bz_eTeamType)1) != 0);
		green = (bz_getTeamPlayerLimit((bz_eTeamType)2) != 0);
		blue = (bz_getTeamPlayerLimit((bz_eTeamType)3) != 0);
		purple = (bz_getTeamPlayerLimit((bz_eTeamType)4) != 0);
		
		// kick player after this many illegal grabs
		illegalGrabThreshold = 3;
		// reset count if time since last grab is more than this many seconds
		illegalGrabTimeout = 60;

		obsMsg = false;
		head = NULL;

		mem = 0;
	}

	int RvG;	//  1: left is strong (ie, red > green)
	int RvB;	//  0: teams are even (ie, red == blue)
	int RvP;	// -1: right is strong (ie, purple > red)
	int GvB;	
	int GvP;
	int BvP;
	bool red;
	bool green;
	bool blue;
	bool purple;
	PlayerInfo *head;
	int illegalGrabThreshold;
	int illegalGrabTimeout;
	int mem;

	bool fairCTFEnabled;
	bool notifiedCTFOK_RG;
	bool notifiedCTFOK_RB;
	bool notifiedCTFOK_RP;
	bool notifiedCTFOK_GB;
	bool notifiedCTFOK_GP;
	bool notifiedCTFOK_BP;
	bool fairCTF;
	bool kickEnabled;
	bool obsMsg;

	virtual void process ( bz_EventData *eventData );
  virtual bool handle ( int playerID, bzApiString command, bzApiString message, bzAPIStringList *param );
	void update(int dir, int team);
	void notify();
};

FCTF ctf;

// use the input to set illegalGrabThreshold
int ConvertToInt(std::string inmessage) {
	int messagelength = (int)inmessage.length();
	if (messagelength > 0 && messagelength < 3){
		int messagevalue = 0;
		int tens = 1;
		for ( int i = (messagelength - 1); i >= 0; i-- ){
			if (inmessage[i] < '0' || inmessage[i] > '9')  // got something other than a number
				return -1; 
			tens *= 10;
			messagevalue +=  (int) ((((double)inmessage[i] - '0') / 10) * tens);
		}
		// we're ok so far, check limits:
		if (messagevalue >= 0 && messagevalue <= 99)
			return messagevalue;
	}
	return -1;
}

BZF_PLUGIN_CALL int bz_Load ( const char* commandLine ) {
  bz_debugMessage(4,"Fair4CTF plugin loaded");

  std::string parameter = commandLine;
  int kickThreshold = ConvertToInt(parameter);
  
  if (kickThreshold != -1) //if it is -1, leave set to the default
	  ctf.illegalGrabThreshold = kickThreshold;

  bz_registerEvent(bz_ePlayerJoinEvent, &ctf);
	bz_registerEvent(bz_ePlayerPartEvent, &ctf);
  bz_registerEvent(bz_eFlagGrabbedEvent, &ctf);
	bz_registerEvent(bz_eTickEvent, &ctf);
  bz_registerCustomSlashCommand("ctf", &ctf);
  return 0;
}

BZF_PLUGIN_CALL int bz_Unload ( void ) {
  bz_debugMessage(4,"Fair4CTF plugin unloaded");

  bz_removeEvent(bz_ePlayerJoinEvent, &ctf);
  bz_removeEvent(bz_ePlayerPartEvent, &ctf);
	bz_removeEvent(bz_eFlagGrabbedEvent, &ctf);
	bz_removeEvent(bz_eTickEvent, &ctf);

  bz_removeCustomSlashCommand("ctf");
  return 0;
}


// check for fairness between each pair of teams:
void FCTF::update(int dir, int team) {
	float TeamRatioTolerance = 0.25; // if not same size, at least 3 versus 4 or better
	
	// compare teams in pairs, ratio is (team difference)/(larger team)
	float RatioRG = 0;
	float RatioRB = 0;
	float RatioRP = 0;
	float RatioGB = 0;
	float RatioBP = 0;
	float RatioGP = 0;
	// get current team sizes
	float RS = (float)bz_getTeamCount(eRedTeam);
	float GS = (float)bz_getTeamCount(eGreenTeam);
	float BS = (float)bz_getTeamCount(eBlueTeam);
	float PS = (float)bz_getTeamCount(ePurpleTeam);
	// subtract the departing player
	if ( dir == -1 ) {
		switch(team) {
			case eRedTeam: 
				RS--;
				break;
			case eGreenTeam:
				GS--;
				break;
			case eBlueTeam:
				BS--;
				break;
			case ePurpleTeam:
				PS--;
				break;
		}
	}

	// calculate ratios
	if (std::max(RS, GS) != 0 && red && green)
		RatioRG = (RS-GS)/std::max(RS, GS);
	if (std::max(RS, BS) != 0 && red && blue)
		RatioRB = (RS-BS)/std::max(RS, BS);
	if (std::max(RS, PS) != 0 && red && purple)
		RatioRP = (RS-PS)/std::max(RS, PS);
	if (std::max(GS, BS) != 0 && green && blue)
		RatioGB = (GS-BS)/std::max(GS, BS);
	if (std::max(GS, PS) != 0 && green && purple)
		RatioGP = (GS-PS)/std::max(GS, PS);
	if (std::max(BS, PS) != 0 && blue && purple)
		RatioBP = (BS-PS)/std::max(BS, PS);
	
	// use ratios and tolerance to determine if the pair is fair
	// if (AvB == 1) A is stronger than B
	// if (AvB == 0) A and B are close enough to be even
	// if (AvB == -1) B is stronger than A
	if (fabs(RatioRG) <= TeamRatioTolerance)
		ctf.RvG = 0;
	else {
		if (RatioRG > 0)
			RvG = 1;
		else
			RvG = -1;
	}
	if (fabs(RatioRB) <= TeamRatioTolerance)
		RvB = 0;
	else {
		if (RatioRB > 0)
			RvB = 1;
		else
			RvB = -1;
	}
	if (fabs(RatioRP) <= TeamRatioTolerance)
		RvP = 0;
	else {
		if (RatioRP > 0)
			RvP = 1;
		else
			RvP = -1;
	}
	if (fabs(RatioGB) <= TeamRatioTolerance)
		GvB = 0;
	else {
		if (RatioGB > 0)
			GvB = 1;
		else
			GvB = -1;
	}
	if (fabs(RatioGP) <= TeamRatioTolerance)
		GvP = 0;
	else {
		if (RatioGP > 0)
			GvP = 1;
		else
			GvP = -1;
	}
	if (fabs(RatioBP) <= TeamRatioTolerance)
		BvP = 0;
	else {
		if (RatioBP > 0)
			BvP = 1;
		else
			BvP = -1;
	}

}



// notify both teams & observers when CTF status changes
void FCTF::notify() {
	std::string obsOn, obsOff;
	obsOn = "";
	obsOff = "";

	// check/notify team balance changes
	// if fair ctf is disabled, no need to check/notify about team balance changes:

	// messages to allow CTF
	if (RvG == 0 && !notifiedCTFOK_RG  && red && green) { // red vs green is OK
		bz_sendTextMessagef (BZ_SERVER, eRedTeam, "Capture the flag enabled for Red vs Green - teams are fair enough.");
		bz_sendTextMessagef (BZ_SERVER, eGreenTeam, "Capture the flag enabled for Green vs Red - teams are fair enough.");
		obsOn += "Red vs Green, ";
		notifiedCTFOK_RG = true;
	}
	if (RvB == 0 && !notifiedCTFOK_RB  && red && blue) { // red vs blue is OK
		bz_sendTextMessagef (BZ_SERVER, eRedTeam, "Capture the flag enabled for Red vs Blue - teams are fair enough.");
		bz_sendTextMessagef (BZ_SERVER, eBlueTeam, "Capture the flag enabled for Blue vs Red - teams are fair enough.");
		obsOn += "Red vs Blue, ";
		notifiedCTFOK_RB = true;
	}
	if (RvP == 0 && !notifiedCTFOK_RP  && red && purple) { // red vs purple is OK
		bz_sendTextMessagef (BZ_SERVER, eRedTeam, "Capture the flag enabled for Red vs Purple - teams are fair enough.");
		bz_sendTextMessagef (BZ_SERVER, ePurpleTeam, "Capture the flag enabled for Purple vs Red - teams are fair enough.");
		obsOn += "Red vs Purple, ";
		notifiedCTFOK_RP = true;
	}
	if (GvB == 0 && !notifiedCTFOK_GB  && green && blue) { // green vs blue is OK
		bz_sendTextMessagef (BZ_SERVER, eGreenTeam, "Capture the flag enabled for Green vs Blue - teams are fair enough.");
		bz_sendTextMessagef (BZ_SERVER, eBlueTeam, "Capture the flag enabled for Blue vs Green - teams are fair enough.");
		obsOn += "Green vs Blue, ";
		notifiedCTFOK_GB = true;
	}
	if (GvP == 0 && !notifiedCTFOK_GP  && green && purple) { // green vs purple is OK
		bz_sendTextMessagef (BZ_SERVER, eGreenTeam, "Capture the flag enabled for Green vs Purple - teams are fair enough.");
		bz_sendTextMessagef (BZ_SERVER, ePurpleTeam, "Capture the flag enabled for Purple vs Green - teams are fair enough.");
		obsOn += "Green vs Purple, ";
		notifiedCTFOK_GP = true;
	}
	if (BvP == 0 && !notifiedCTFOK_BP  && blue && purple) { // blue vs purple is OK
		bz_sendTextMessagef (BZ_SERVER, eBlueTeam, "Capture the flag enabled for Blue vs Purple - teams are fair enough.");
		bz_sendTextMessagef (BZ_SERVER, ePurpleTeam, "Capture the flag enabled for Purple vs Blue - teams are fair enough.");
		obsOn += "Blue vs Purple, ";
		notifiedCTFOK_BP = true;
	}
	
	// messages to disable CTF
	if (notifiedCTFOK_RG  && red && green) { // red vs green is uneven
		if (RvG > 0){
			bz_sendTextMessagef (BZ_SERVER, eRedTeam, "Capture the flag disabled against Green - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, eGreenTeam, "Green flag is safe from Red - teams are uneven.");
			obsOff += "Red vs Green, ";
			notifiedCTFOK_RG = false;
		}
		if (RvG < 0){
			bz_sendTextMessagef (BZ_SERVER, eGreenTeam, "Capture the flag disabled against Red - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, eRedTeam, "Red flag is safe from Green - teams are uneven.");
			obsOff += "Green vs Red, ";
			notifiedCTFOK_RG = false;
		}
	}
	if (notifiedCTFOK_RB  && red && blue) { // red vs blue is uneven
		if (RvB > 0){
			bz_sendTextMessagef (BZ_SERVER, eRedTeam, "Capture the flag disabled against Blue - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, eBlueTeam, "Blue flag is safe from Red - teams are uneven.");
			obsOff += "Red vs Blue, ";
			notifiedCTFOK_RB = false;
		}
		if (RvB < 0){
			bz_sendTextMessagef (BZ_SERVER, eBlueTeam, "Capture the flag disabled against Red - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, eRedTeam, "Red flag is safe from Blue - teams are uneven.");
			obsOff = obsOff+ "Blue vs Red, ";
			notifiedCTFOK_RB = false;
		}
	}
	if (notifiedCTFOK_RP  && red && purple) { // red vs purple is uneven
		if (RvP > 0){
			bz_sendTextMessagef (BZ_SERVER, eRedTeam, "Capture the flag disabled against Purple - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, ePurpleTeam, "Purple flag is safe from Red - teams are uneven.");
			obsOff += "Red vs Purple, ";
			notifiedCTFOK_RP = false;
		}
		if (RvP < 0){
			bz_sendTextMessagef (BZ_SERVER, ePurpleTeam, "Capture the flag disabled against Red - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, eRedTeam, "Red flag is safe from Purple - teams are uneven.");
			obsOff += "Purple vs Red, ";
			notifiedCTFOK_RP = false;
		}
	}
	if (notifiedCTFOK_GB  && green && blue) { // green vs blue is uneven
		if (GvB > 0){
			bz_sendTextMessagef (BZ_SERVER, eGreenTeam, "Capture the flag disabled against Blue - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, eBlueTeam, "Blue flag is safe from Green - teams are uneven.");
			obsOff += "Green vs Blue, ";
			notifiedCTFOK_GB = false;
		}
		if (GvB < 0){
			bz_sendTextMessagef (BZ_SERVER, eBlueTeam, "Capture the flag disabled against Green - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, eGreenTeam, "Green flag is safe from Blue - teams are uneven.");
			obsOff += "Blue vs Green, ";
			notifiedCTFOK_GB = false;
		}
	}
	if (notifiedCTFOK_GP  && green && purple) { // green vs purple is uneven
		if (GvP > 0){
			bz_sendTextMessagef (BZ_SERVER, eGreenTeam, "Capture the flag disabled against Purple - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, ePurpleTeam, "Purple flag is safe from Green - teams are uneven.");
			obsOff += "Green vs Purple, ";
			notifiedCTFOK_GP = false;
		}
		if (GvP < 0){
			bz_sendTextMessagef (BZ_SERVER, ePurpleTeam, "Capture the flag disabled against Green - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, eGreenTeam, "Green flag is safe from Purple - teams are uneven.");
			obsOff += "Purple vs Green, ";
			notifiedCTFOK_GP = false;
		}
	}
	if (notifiedCTFOK_BP  && blue && purple) { // blue vs purple is uneven
		if (BvP > 0){
			bz_sendTextMessagef (BZ_SERVER, eBlueTeam, "Capture the flag disabled against Purple - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, ePurpleTeam, "Purple flag is safe from Blue - teams are uneven.");
			obsOff += "Blue vs Purple, ";
			notifiedCTFOK_BP = false;
		}
		if (BvP < 0){
			bz_sendTextMessagef (BZ_SERVER, ePurpleTeam, "Capture the flag disabled against Blue - teams are uneven.");
			bz_sendTextMessagef (BZ_SERVER, eBlueTeam, "Blue flag is safe from Purple - teams are uneven.");
			obsOff += "Purple vs Blue, ";
			notifiedCTFOK_BP = false;
		}
	}
	if (obsMsg) {
		std::string output = "";
		if (obsOn.length() != 0) {
			obsOn = obsOn.substr(0, obsOn.length() - 2);
			output += "CTF enabled for: " + obsOn + ".   ";
		}
		if (obsOff.length() != 0) {
			obsOff = obsOff.substr(0, obsOff.length() - 2);
			output += "CTF disabled for: " + obsOff + ".";
		}
		if (output.length() != 0)
			bz_sendTextMessagef(BZ_SERVER, eObservers, output.c_str());
	}

	return;
}



// notifies joining players the current ctf status (allowed/disallowed)
void FCTF::process ( bz_EventData *eventData ) {
	if (!fairCTFEnabled) return;
	bool grab = true;
	switch(eventData->eventType) {
		case bz_ePlayerJoinEvent:
			grab = false;
			update(1, (( bz_PlayerJoinPartEventData*)eventData)->team);
			notify();

			PlayerInfo *add, *spot;
			double time;
			time = bz_getCurrentTime();
			add = (PlayerInfo*) malloc(sizeof(PlayerInfo));
			mem++;
			add->id = (( bz_PlayerJoinPartEventData*)eventData)->playerID;
			add->grabs = 0;
			add->lastGrab = time;
			add->next = NULL;

			if (head == NULL) {  // if head is null, add player to first slot
				head = add;
			} else {			// find the right spot to add the playerID
				spot = head;
				if (spot->id > add->id) {		//add to beginning
					add->next = head;
					head = add;
				}
				while (spot->next != NULL) {
					if (spot->next->id > add->id) {		// insert here
						add->next = spot->next;
						spot->next = add;
						break;
					}
					spot = spot->next;
				}
				if (spot->next == NULL) {		// add to end
					spot->next = add;
				}
			}
			break;
		case bz_ePlayerPartEvent:
			grab = false;
			// remove player from the bad list
			int playerID;
			playerID = (( bz_PlayerJoinPartEventData*)eventData)->playerID;
			PlayerInfo *data, *temp;
			if (head == NULL) break;
			if (head->id == playerID) {
				temp = head;
				head = head->next;
				free(temp);
				mem--;
			} else {
				for (data = head; data->next != NULL; data = data->next) {
					if (data->next->id == playerID) {
						// remove this one
						temp = data->next;
						data->next = data->next->next;
						free(temp);
						mem--;
						break;
					}
				}
			}
			update(-1, (( bz_PlayerJoinPartEventData*)eventData)->team);
			notify();
			break;
		case bz_eFlagGrabbedEvent:
			return;
	}
		// check all players for illegal flag carries
	bzAPIIntList *playerList = bz_newIntList();
	int numPlayers, i;
	bool logForKick = false;

	bz_getPlayerIndexList(playerList);
	numPlayers = playerList->size();
	bz_PlayerRecord *record;
	for (i=0; i<numPlayers; i++) {

		int playerID = playerList->get(i);

		record = bz_getPlayerByIndex(playerID);
		if (record == NULL) continue;
		int team = record->team;
		bz_freePlayerRecord (record);

		const char* FlagHeld = bz_getPlayerFlag(playerID);
		if (FlagHeld == NULL) continue;

		int teamFlag = 0;
		if (strcmp(FlagHeld, "R*") == 0) teamFlag = eRedTeam;
		if (strcmp(FlagHeld, "G*") == 0) teamFlag = eGreenTeam;
		if (strcmp(FlagHeld, "B*") == 0) teamFlag = eBlueTeam;
		if (strcmp(FlagHeld, "P*") == 0) teamFlag = ePurpleTeam;


		// set logForKick to true if the flag grabbed belongs to a weaker team
		switch (team) {
			case eRedTeam:
				logForKick = ((RvG == 1) && (teamFlag==eGreenTeam)) ||
							 ((RvB == 1) && (teamFlag==eBlueTeam)) ||
							 ((RvP == 1) && (teamFlag==ePurpleTeam));
				break;
			case eGreenTeam:
				logForKick = ((RvG == -1) && (teamFlag==eRedTeam)) ||
							 ((GvB == 1) && (teamFlag==eBlueTeam)) ||
							 ((GvP == 1) && (teamFlag==ePurpleTeam));
				break;
			case eBlueTeam:
				logForKick = ((RvB == -1) && (teamFlag==eRedTeam)) ||
							 ((GvB == -1) && (teamFlag==eGreenTeam)) ||
							 ((BvP == 1) && (teamFlag==ePurpleTeam));
				break;
			case ePurpleTeam:
				logForKick = ((RvP == -1) && (teamFlag==eRedTeam)) ||
							 ((GvP == -1) && (teamFlag==eGreenTeam)) ||
							 ((BvP == -1) && (teamFlag==eBlueTeam));
				break;
		}
		if (logForKick) {
			bz_removePlayerFlag ( playerID );
			bz_sendTextMessagef(BZ_SERVER, playerID, "No CTF for %s (uneven teams).  You will be %s if you persist", FlagHeld, (kickEnabled)?"kicked":"killed");

			if (grab) {
				// increment the grab count
				PlayerInfo *spot;
				for (spot = head; spot != NULL; spot = spot->next) {
					if ( spot->lastGrab + illegalGrabTimeout < bz_getCurrentTime() )
						spot->grabs = 0;
					if (spot->id == playerID) {
						spot->grabs++;
						spot->lastGrab = bz_getCurrentTime();
						break;
					}
				}
				if (spot == NULL) return;

				if (spot->grabs >= illegalGrabThreshold) {
					spot->grabs = 0;
					if (kickEnabled)
						bz_kickUser(playerID, "Repeatedly grabbing a weaker team's flag when CTF is unfair", true);
					else
						bz_killPlayer(playerID, true, BZ_SERVER);
				}

				break;
			}
		}
	}
	return;
}



bool FCTF::handle ( int playerID, bzApiString _command, bzApiString _message, bzAPIStringList * /*_param*/ ) {

	std::string command = _command.c_str();
	std::string message = _message.c_str();

	bz_PlayerRecord *record;
	record = bz_getPlayerByIndex(playerID);
	const int team = record->team;
	bz_freePlayerRecord (record);


	if (!bz_hasPerm(playerID, "FAIRCTF")) {
		bz_sendTextMessagef(BZ_SERVER, playerID, "You do not have permission to run the /ctf command.");
	} else {
		if (message == "on") {
			if (!fairCTFEnabled) {
				bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Fair4CTF plugin enabled.");
				fairCTFEnabled = true;
			} else 
				bz_sendTextMessagef(BZ_SERVER, playerID, "Fair4CTF plugin is already running.");
		} else if (message == "off") {
			if (fairCTFEnabled) {
				bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Fair4CTF plugin disabled.");
				fairCTFEnabled = false;
			} else 
				bz_sendTextMessagef(BZ_SERVER, playerID, "Fair4CTF plugin is already stopped.");
		} else if (message == "kick") {
			if (!kickEnabled) {
				bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Fair4CTF set to /kick on illegal flag grabs.");
				kickEnabled = true;
			} else 
				bz_sendTextMessagef(BZ_SERVER, playerID, "Fair4CTF is already set to /kick.");
		} else	if (message == "kill") {
			if (kickEnabled) {
				bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Fair4CTF set to /kill on illegal flag grabs.");
				kickEnabled = false;
			} else 
				bz_sendTextMessagef(BZ_SERVER, playerID, "Fair4CTF is already set to /kill.");
		} else if (message == "msgon") {
			if (!obsMsg) {
				bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Fair4CTF observer messages are enabled.");
				obsMsg = true;
			} else 
				bz_sendTextMessagef(BZ_SERVER, playerID, "Fair4CTF observer messages are already enabled.");
		} else if (message == "msgoff") {
			if (obsMsg) {
				bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Fair4CTF observer messages are disabled.");
				obsMsg = false;
			} else 
				bz_sendTextMessagef(BZ_SERVER, playerID, "Fair4CTF observer messages are already disabled.");
		} else if (message == "list") {
			PlayerInfo *data;
			bz_sendTextMessagef(BZ_SERVER, playerID, "%12s%8s%12s    %d", "callsign", "grabs", "time", mem);
			for (data = head; data != NULL; data = data->next) {
				if ( data->lastGrab + illegalGrabTimeout < bz_getCurrentTime() )
					data->grabs = 0;
				record = bz_getPlayerByIndex(data->id);
				if (record == NULL) 
					bz_sendTextMessagef(BZ_SERVER, playerID, "Null player record at playerID %d", data->id);
				else
					bz_sendTextMessagef(BZ_SERVER, playerID, "%12s%8d%12.3f", record->callsign.c_str(), data->grabs, data->lastGrab);
				bz_freePlayerRecord(record);
			}
		} else if (message.length() >= 5 && message.substr(0,5) == "grabs") {
			if (message.length() <= 6)
				bz_sendTextMessagef(BZ_SERVER, playerID, "Illegal grab limit: %d", illegalGrabThreshold);
			else {
				int param = ConvertToInt(message.substr(6, message.length()-6));
				if (param == -1) 
					bz_sendTextMessagef(BZ_SERVER, playerID, "Illegal grab limit: %d", illegalGrabThreshold);
				else {
					illegalGrabThreshold = param;
					bz_sendTextMessagef(BZ_SERVER, BZ_ALLUSERS, "Illegal grab limit set to %d", illegalGrabThreshold);
				}
			}
		} else
			bz_sendTextMessagef(BZ_SERVER, playerID, "Usage: /ctf (on|off|kick|kill|msgon|msgoff|grabs [#])");
		}
	return true;
}



// Local Variables: ***
// mode:C++ ***
// tab-width: 8 ***
// c-basic-offset: 2 ***
// indent-tabs-mode: t ***
// End: ***
// ex: shiftwidth=2 tabstop=8

